@@ -24,10 +24,13 @@
  *
  * The approach taken here is to invoke pg_resetxlog with -n option
  * and then pipe its output. With little string parsing we get the
- * pg_control data.
+ * pg_control data.  pg_resetxlog cannot be run while the server is running
+ * so we use pg_controldata;  pg_controldata doesn't provide all the fields
+ * we need to actually perform the migration, but it provides enough for
+ * check mode.
  */
 void
-get_control_data(migratorContext *ctx, ClusterInfo *cluster)
+get_control_data(migratorContext *ctx, ClusterInfo *cluster, bool live_check)
 {
 	char		cmd[MAXPGPATH];
 	char		bufin[MAX_STRING];
@@ -49,7 +52,7 @@ get_control_data(migratorContext *ctx, ClusterInfo *cluster)
 	bool		got_date_is_int = false;
 	bool		got_float8_pass_by_value = false;
 	char	   *lang = NULL;
-
+					
 	/* Because we test the pg_resetxlog output strings, it has to be in English. */
 	if (getenv("LANG"))
 		lang = pg_strdup(ctx, getenv("LANG"));
@@ -58,8 +61,10 @@ get_control_data(migratorContext *ctx, ClusterInfo *cluster)
 #else
 	SetEnvironmentVariableA("LANG", "C");
 #endif
-	sprintf(cmd, SYSTEMQUOTE "\"%s/pg_resetxlog\" -n \"%s\"" SYSTEMQUOTE,
-			cluster->bindir, cluster->pgdata);
+	sprintf(cmd, SYSTEMQUOTE "\"%s/%s \"%s\"" SYSTEMQUOTE,
+			cluster->bindir,
+			live_check ? "pg_controldata\"" : "pg_resetxlog\" -n",
+			cluster->pgdata);
 	fflush(stdout);
 	fflush(stderr);
 
@@ -115,7 +120,7 @@ get_control_data(migratorContext *ctx, ClusterInfo *cluster)
 			p = strchr(p, ':');
 
 			if (p == NULL || strlen(p) <= 1)
-				pg_log(ctx, PG_FATAL, "%d: pg_resetxlog  problem\n", __LINE__);
+				pg_log(ctx, PG_FATAL, "%d: pg_resetxlog problem\n", __LINE__);
 
 			p++;				/* removing ':' char */
 			cluster->controldata.ctrl_ver = (uint32) atol(p);
@@ -125,7 +130,7 @@ get_control_data(migratorContext *ctx, ClusterInfo *cluster)
 			p = strchr(p, ':');
 
 			if (p == NULL || strlen(p) <= 1)
-				pg_log(ctx, PG_FATAL, "%d: pg_resetxlog  problem\n", __LINE__);
+				pg_log(ctx, PG_FATAL, "%d: controldata retrieval problem\n", __LINE__);
 
 			p++;				/* removing ':' char */
 			cluster->controldata.cat_ver = (uint32) atol(p);
@@ -137,7 +142,7 @@ get_control_data(migratorContext *ctx, ClusterInfo *cluster)
 			p = strchr(p, ':');
 
 			if (p == NULL || strlen(p) <= 1)
-				pg_log(ctx, PG_FATAL, "%d: pg_resetxlog  problem\n", __LINE__);
+				pg_log(ctx, PG_FATAL, "%d: controldata retrieval problem\n", __LINE__);
 
 			p++;				/* removing ':' char */
 			cluster->controldata.logid = (uint32) atol(p);
@@ -150,7 +155,7 @@ get_control_data(migratorContext *ctx, ClusterInfo *cluster)
 			p = strchr(p, ':');
 
 			if (p == NULL || strlen(p) <= 1)
-				pg_log(ctx, PG_FATAL, "%d: pg_resetxlog  problem\n", __LINE__);
+				pg_log(ctx, PG_FATAL, "%d: controldata retrieval problem\n", __LINE__);
 
 			p++;				/* removing ':' char */
 			cluster->controldata.nxtlogseg = (uint32) atol(p);
@@ -161,7 +166,7 @@ get_control_data(migratorContext *ctx, ClusterInfo *cluster)
 			p = strchr(p, ':');
 
 			if (p == NULL || strlen(p) <= 1)
-				pg_log(ctx, PG_FATAL, "%d: pg_resetxlog  problem\n", __LINE__);
+				pg_log(ctx, PG_FATAL, "%d: controldata retrieval problem\n", __LINE__);
 
 			p++;				/* removing ':' char */
 			cluster->controldata.chkpnt_tli = (uint32) atol(p);
@@ -175,7 +180,7 @@ get_control_data(migratorContext *ctx, ClusterInfo *cluster)
 				op = strchr(p, ':');
 
 			if (op == NULL || strlen(op) <= 1)
-				pg_log(ctx, PG_FATAL, "%d: pg_resetxlog  problem\n", __LINE__);
+				pg_log(ctx, PG_FATAL, "%d: controldata retrieval problem\n", __LINE__);
 
 			op++;				/* removing ':' char */
 			cluster->controldata.chkpnt_nxtxid = (uint32) atol(op);
@@ -186,7 +191,7 @@ get_control_data(migratorContext *ctx, ClusterInfo *cluster)
 			p = strchr(p, ':');
 
 			if (p == NULL || strlen(p) <= 1)
-				pg_log(ctx, PG_FATAL, "%d: pg_resetxlog  problem\n", __LINE__);
+				pg_log(ctx, PG_FATAL, "%d: controldata retrieval problem\n", __LINE__);
 
 			p++;				/* removing ':' char */
 			cluster->controldata.chkpnt_nxtoid = (uint32) atol(p);
@@ -197,7 +202,7 @@ get_control_data(migratorContext *ctx, ClusterInfo *cluster)
 			p = strchr(p, ':');
 
 			if (p == NULL || strlen(p) <= 1)
-				pg_log(ctx, PG_FATAL, "%d: pg_resetxlog  problem\n", __LINE__);
+				pg_log(ctx, PG_FATAL, "%d: controldata retrieval problem\n", __LINE__);
 
 			p++;				/* removing ':' char */
 			cluster->controldata.align = (uint32) atol(p);
@@ -208,7 +213,7 @@ get_control_data(migratorContext *ctx, ClusterInfo *cluster)
 			p = strchr(p, ':');
 
 			if (p == NULL || strlen(p) <= 1)
-				pg_log(ctx, PG_FATAL, "%d: pg_resetxlog  problem\n", __LINE__);
+				pg_log(ctx, PG_FATAL, "%d: controldata retrieval problem\n", __LINE__);
 
 			p++;				/* removing ':' char */
 			cluster->controldata.blocksz = (uint32) atol(p);
@@ -219,7 +224,7 @@ get_control_data(migratorContext *ctx, ClusterInfo *cluster)
 			p = strchr(p, ':');
 
 			if (p == NULL || strlen(p) <= 1)
-				pg_log(ctx, PG_FATAL, "%d: pg_resetxlog  problem\n", __LINE__);
+				pg_log(ctx, PG_FATAL, "%d: controldata retrieval problem\n", __LINE__);
 
 			p++;				/* removing ':' char */
 			cluster->controldata.largesz = (uint32) atol(p);
@@ -230,7 +235,7 @@ get_control_data(migratorContext *ctx, ClusterInfo *cluster)
 			p = strchr(p, ':');
 
 			if (p == NULL || strlen(p) <= 1)
-				pg_log(ctx, PG_FATAL, "%d: pg_resetxlog  problem\n", __LINE__);
+				pg_log(ctx, PG_FATAL, "%d: controldata retrieval problem\n", __LINE__);
 
 			p++;				/* removing ':' char */
 			cluster->controldata.walsz = (uint32) atol(p);
@@ -241,7 +246,7 @@ get_control_data(migratorContext *ctx, ClusterInfo *cluster)
 			p = strchr(p, ':');
 
 			if (p == NULL || strlen(p) <= 1)
-				pg_log(ctx, PG_FATAL, "%d: pg_resetxlog  problem\n", __LINE__);
+				pg_log(ctx, PG_FATAL, "%d: controldata retrieval problem\n", __LINE__);
 
 			p++;				/* removing ':' char */
 			cluster->controldata.walseg = (uint32) atol(p);
@@ -252,7 +257,7 @@ get_control_data(migratorContext *ctx, ClusterInfo *cluster)
 			p = strchr(p, ':');
 
 			if (p == NULL || strlen(p) <= 1)
-				pg_log(ctx, PG_FATAL, "%d: pg_resetxlog  problem\n", __LINE__);
+				pg_log(ctx, PG_FATAL, "%d: controldata retrieval problem\n", __LINE__);
 
 			p++;				/* removing ':' char */
 			cluster->controldata.ident = (uint32) atol(p);
@@ -263,7 +268,7 @@ get_control_data(migratorContext *ctx, ClusterInfo *cluster)
 			p = strchr(p, ':');
 
 			if (p == NULL || strlen(p) <= 1)
-				pg_log(ctx, PG_FATAL, "%d: pg_resetxlog  problem\n", __LINE__);
+				pg_log(ctx, PG_FATAL, "%d: controldata retrieval problem\n", __LINE__);
 
 			p++;				/* removing ':' char */
 			cluster->controldata.index = (uint32) atol(p);
@@ -274,7 +279,7 @@ get_control_data(migratorContext *ctx, ClusterInfo *cluster)
 			p = strchr(p, ':');
 
 			if (p == NULL || strlen(p) <= 1)
-				pg_log(ctx, PG_FATAL, "%d: pg_resetxlog  problem\n", __LINE__);
+				pg_log(ctx, PG_FATAL, "%d: controldata retrieval problem\n", __LINE__);
 
 			p++;				/* removing ':' char */
 			cluster->controldata.toast = (uint32) atol(p);
@@ -285,7 +290,7 @@ get_control_data(migratorContext *ctx, ClusterInfo *cluster)
 			p = strchr(p, ':');
 
 			if (p == NULL || strlen(p) <= 1)
-				pg_log(ctx, PG_FATAL, "%d: pg_resetxlog  problem\n", __LINE__);
+				pg_log(ctx, PG_FATAL, "%d: controldata retrieval problem\n", __LINE__);
 
 			p++;				/* removing ':' char */
 			cluster->controldata.date_is_int = strstr(p, "64-bit integers") != NULL;
@@ -296,7 +301,7 @@ get_control_data(migratorContext *ctx, ClusterInfo *cluster)
 			p = strchr(p, ':');
 
 			if (p == NULL || strlen(p) <= 1)
-				pg_log(ctx, PG_FATAL, "%d: pg_resetxlog  problem\n", __LINE__);
+				pg_log(ctx, PG_FATAL, "%d: controldata retrieval problem\n", __LINE__);
 
 			p++;				/* removing ':' char */
 			/* used later for /contrib check */
@@ -309,7 +314,7 @@ get_control_data(migratorContext *ctx, ClusterInfo *cluster)
 			p = strchr(p, ':');
 
 			if (p == NULL || strlen(p) <= 1)
-				pg_log(ctx, PG_FATAL, "%d: pg_resetxlog  problem\n", __LINE__);
+				pg_log(ctx, PG_FATAL, "%d: controldata retrieval problem\n", __LINE__);
 
 			p++;				/* removing ':' char */
 			/* skip leading spaces and remove trailing newline */
@@ -324,7 +329,7 @@ get_control_data(migratorContext *ctx, ClusterInfo *cluster)
 			p = strchr(p, ':');
 
 			if (p == NULL || strlen(p) <= 1)
-				pg_log(ctx, PG_FATAL, "%d: pg_resetxlog  problem\n", __LINE__);
+				pg_log(ctx, PG_FATAL, "%d: controldata retrieval problem\n", __LINE__);
 
 			p++;				/* removing ':' char */
 			/* skip leading spaces and remove trailing newline */
@@ -361,7 +366,10 @@ get_control_data(migratorContext *ctx, ClusterInfo *cluster)
 	}
 
 	/* verify that we got all the mandatory pg_control data */
-	if (!got_xid || !got_oid || !got_log_id || !got_log_seg || !got_tli ||
+	if (!got_xid || !got_oid ||
+		(!live_check && !got_log_id) ||
+		(!live_check && !got_log_seg) ||
+		!got_tli ||
 		!got_align || !got_blocksz || !got_largesz || !got_walsz ||
 		!got_walseg || !got_ident || !got_index || !got_toast ||
 		!got_date_is_int || !got_float8_pass_by_value)
@@ -375,10 +383,10 @@ get_control_data(migratorContext *ctx, ClusterInfo *cluster)
 		if (!got_oid)
 			pg_log(ctx, PG_REPORT, "  latest checkpoint next OID\n");
 
-		if (!got_log_id)
+		if (!live_check && !got_log_id)
 			pg_log(ctx, PG_REPORT, "  first log file ID after reset\n");
 
-		if (!got_log_seg)
+		if (!live_check && !got_log_seg)
 			pg_log(ctx, PG_REPORT, "  first log file segment after reset\n");
 
 		if (!got_tli)
